from analyst_klondike.features.code.code_runner.runner import CodeRunner


_runner_cases: list[dict[str, object]] = [
    {
        "n": 1,
        "expected": "1 просмотр"
    },
    {
        "n": 2,
        "expected": "2 просмотра"
    },
    {
        "n": 2,
        "expected": "2 просмотра"
    },
    {
        "n": 3,
        "expected": "3 просмотра"
    },
    {
        "n": 10,
        "expected": "10 просмотров"
    },
    {
        "n": 21,
        "expected": "21 просмотр"
    }
]


def test_run_code() -> None:
    code = """\
def solution(n: int) -> str:
    view_dict = {
        1: "1 просмотр",
        2: "2 просмотра",
        3: "3 просмотра",
        4: "4 просмотра",
    }
    if n <= 4:
        return view_dict[n]
    return f"{n} просмотров"
    """
    result = CodeRunner().run_code(_runner_cases, code)
    assert result is not None
    assert len(list(result.all_cases)) > 0


def test_overriding_print() -> None:
    code = """\

def solution(n: int) -> str:
    view_dict = {
        1: "1 просмотр",
        2: "2 просмотра",
        3: "3 просмотра",
        4: "4 просмотра",
    }
    if n <= 4:
        return view_dict[n]
    print("hello")
    print("world")
    return f"{n} просмотров"
    """
    code_runner = CodeRunner()
    result = code_runner.run_code(_runner_cases, code)
    assert result is not None
    assert "hello" in result.printed_lines
    assert "world" in result.printed_lines


def test_update_dict():
    initial_dict = {
        "a": 1,
        "b": 2
    }

    initial_dict.update({
        "c": 3,
        "d": 4
    })

    assert "c" in initial_dict



from analyst_klondike.state.app_state import AppState


def current_file_path(state: AppState) -> tuple[str, str]:
    return (
        state.current.opened_file_path,
        state.current.opened_file_name
    )



from dataclasses import dataclass
from os.path import exists
import json
from typing import Any


@dataclass
class UserInfoJson:
    email: str


@dataclass
class QuestionJson:
    id: int
    title: str
    text: str
    code_template: str
    code: str
    test_cases: list[dict[str, Any]]
    is_passed: str


@dataclass
class PythonQuizJson:
    id: str
    title: str
    questions: list[QuestionJson]


@dataclass
class JsonLoadResult:
    user_info: UserInfoJson
    quizes: list[PythonQuizJson]


def get_quiz_json(file_path: str) -> JsonLoadResult:
    if not exists(file_path):
        raise FileExistsError(f"<{file_path}> not found")
    load_result = JsonLoadResult(
        user_info=UserInfoJson(email=""),
        quizes=[]
    )
    with open(file_path, encoding='UTF-8') as f:
        json_data = json.load(f)
        load_result.user_info.email = json_data["user_info"]["email"]
        # map quizes
        for quiz_json in json_data["quizes"]:
            quiz_obj = PythonQuizJson(
                id=quiz_json["id"],
                title=quiz_json["title"],
                questions=[]
            )
            # map questions
            for question_json in quiz_json["questions"]:
                quiz_obj.questions.append(
                    QuestionJson(
                        id=int(question_json["id"]),
                        title=question_json["title"],
                        text=question_json["text"],
                        code_template=question_json["code_template"],
                        code=question_json["code"],
                        test_cases=question_json["test_cases"],
                        is_passed=question_json.get("is_passed", "")
                    )
                )
            load_result.quizes.append(quiz_obj)

    return load_result



from dataclasses import dataclass
import json
from typing import Any

from analyst_klondike.state.app_state import AppState
from analyst_klondike.state.base_action import BaseAction


@dataclass
class SaveAction(BaseAction):
    type = "SAVE_ACTION"


def save_to_json(state: AppState):
    with open(state.current.opened_file_path,
              encoding='utf-8',
              mode='w') as f:
        data = _create_json(state)
        json.dump(data, f, indent=4, ensure_ascii=False)


def _create_json(state: AppState) -> Any:
    def _get_tasks(quiz_id: str):
        return (
            t for t in state.data.tasks.values() if t.quiz_id == quiz_id
        )

    d = {  # type: ignore
        "user_info": {
            "email": state.user_email
        },
        "quizes": [
            {
                "id": quiz.id,
                "title": quiz.title,
                "questions": [
                    {
                        "id": t.id,
                        "title": t.title,
                        "text": t.description,
                        "code_template": t.code_template,
                        "code": t.code,
                        "test_cases": t.test_cases,
                        "is_passed": t.is_passed
                    } for t in _get_tasks(quiz.id)
                ]
            } for quiz in state.data.quizes.values()
        ]
    }
    return d  # type: ignore
